#!/bin/bash
set -e

echo "_______________________________________________________________ "
echo "___ Automatic Maximum Iverter Power Selection for A602/750 ____ "
echo "______________ Automatic Grid Point Selection _________________ "
echo "_______________________________________________________________ "



#--------------------------------------------------------------
#--- Read out PV power of Fronius PV Inverter at ACout      ---
#--- Calculates the average power value with 5 measurements ---
#--------------------------------------------------------------
counter=1
until [ $counter -gt 5 ]
do
	FroniusPwr=$(dbus -y com.victronenergy.system /Ac/PvOnOutput/L1/Power GetValue)
	if [ "$FroniusPwr" = "[]" ]; then 
		FroniusPwr=0
		echo "Fronius PV is offline"
	else
		FroniusPwr="${FroniusPwr%.*}"		 
	fi
		echo "Fronius PV Power measurement $i: $FroniusPwr Watt"
		FroniusPwrSum=$(( $FroniusPwrSum + $FroniusPwr ))
		SolarPwrCalc=$(( $FroniusPwrSum / $counter ))

	if [ $counter -lt 5 ]
	then
		echo "Wait 60 sec. to read out next value"
		sleep 60
	fi
	((counter++))
done

echo "Average Fronius PV Power for calculation: $SolarPwrCalc Watt"
echo " "



#--------------------------------------------------------
#--- Read out PV power of Victron Smart Solar Charger ---
#--------------------------------------------------------
PVChargerPwr=$(dbus -y com.victronenergy.system /Dc/Pv/Power GetValue)
if [ "$PVChargerPwr" = "[]" ]; then 
		PVChargerPwr=0
		echo "PV Charger is offline"
	else	
		PVChargerPwr="${PVChargerPwr%.*}"
		echo "Smart Solar Charger PV Power: $PVChargerPwr Watt"		 
	fi
	

if [ $PVChargerPwr -gt $SolarPwrCalc ]; then
      SolarPwrCalc=$PVChargerPwr
      echo "Use PV Charger Power value instead of Fronius Power value"
fi



#-------------------------------------------------------------
#--- Read out Battery State of Charge and set GridSetpoint ---
#-------------------------------------------------------------
BatterySoC=$(dbus -y com.victronenergy.system /Dc/Battery/Soc GetValue)
BatterySoC="${BatterySoC%.*}"


#--- Calculate Power Offset according Battery State in percent
if [ $BatterySoC -ge 98 ]; then
      echo "Full battery capacity SoC: $BatterySoC percent"
      SoCOffset=2300
      GridSetpoint=-30

elif [ $BatterySoC -ge 95 ]; then
      echo "Very good battery capacity SoC: $BatterySoC percent"
      SoCOffset=2300
      GridSetpoint=-30

elif [ $BatterySoC -ge 90 ]; then
      echo "Good battery capacity SoC: $BatterySoC percent"
      SoCOffset=2300
      GridSetpoint=-20

elif [ $BatterySoC -ge 85 ]; then
      echo "Medium battery capacity SoC: $BatterySoC percent"
      SoCOffset=2300
      GridSetpoint=-20

elif [ $BatterySoC -ge 80 ]; then
      echo "Medium battery capacity SoC: $BatterySoC percent"
      SoCOffset=2300
      GridSetpoint=-20

elif [ $BatterySoC -ge 75 ]; then
      echo "Low battery capacity SoC: $BatterySoC percent"
      SoCOffset=2100
      GridSetpoint=-10

elif [ $BatterySoC -ge 70 ]; then
      echo "Very low battery capacity SoC: $BatterySoc percent"
      SoCOffset=2000
      GridSetpoint=0
else
      echo "Depleted battery capacity SoC: $BatterySoC percent"
      SoCOffset=1000
      GridSetpoint=0
fi

echo "Battery SoCOffset: $SoCOffset Watt"
echo


#-----------------------------------------------
#--- Max Discharge Power Value determination ---
#--- Grid Point Power Value determination ------
#-----------------------------------------------
if [ $SolarPwrCalc -ge 1300 ]; then
      MaxDischargePwr=$((100 + $SoCOffset))
	  GridSetpoint=-50

elif [ $SolarPwrCalc -ge 1200 ]; then
      MaxDischargePwr=$((100 + $SoCOffset))
	  GridSetpoint=-50

elif [ $SolarPwrCalc -ge 1100 ]; then
      MaxDischargePwr=$((100 + $SoCOffset))
	  GridSetpoint=-50

elif [ $SolarPwrCalc -ge 1000 ]; then
      MaxDischargePwr=$((100 + $SoCOffset))
	  GridSetpoint=-50

elif [ $SolarPwrCalc -ge 900 ]; then
      MaxDischargePwr=$((100 + $SoCOffset))
	  GridSetpoint=-40

elif [ $SolarPwrCalc -ge 800 ]; then
      MaxDischargePwr=$((100 + $SoCOffset))
	  GridSetpoint=-40

elif [ $SolarPwrCalc -ge 700 ]; then
      MaxDischargePwr=$((100 + $SoCOffset))
	  GridSetpoint=-30
 
elif [ $SolarPwrCalc -ge 600 ]; then
      MaxDischargePwr=$((100 + $SoCOffset))
	  GridSetpoint=-30
	  	  
elif [ $SolarPwrCalc -ge 500 ]; then
      MaxDischargePwr=$((100 + $SoCOffset))
	  GridSetpoint=-30
	  
elif [ $SolarPwrCalc -ge 400 ]; then
      MaxDischargePwr=$((100 + $SoCOffset))
	  GridSetpoint=-30
	  	  
elif [ $SolarPwrCalc -ge 300 ]; then
      MaxDischargePwr=$((100 + $SoCOffset))
	  
elif [ $SolarPwrCalc -ge 200 ]; then
      MaxDischargePwr=$((100 + $SoCOffset))

else #--- Very low or NO Solar power available (Night)
      MaxDischargePwr=$((100 + $SoCOffset))
fi

#--------------------------
#--- Set ESS Grid Point ---
#--------------------------
dbus -y com.victronenergy.settings /Settings/CGwacs/AcPowerSetPoint SetValue %"$GridSetpoint"
echo "Grid Setpoint: $GridSetpoint Watt"
echo	  


#------------------------------------------------
#--- Set MAX Discharge Power value at VenusGX ---
#------------------------------------------------
dbus -y com.victronenergy.settings /Settings/CGwacs/MaxDischargePower SetValue "$MaxDischargePwr"


#---------------
#---LOG File ---
#---------------
Logtext="ESS Max.Inverter Power: $MaxDischargePwr W / Grid Setpoint: $GridSetpoint W / Battery SoC: $BatterySoC percent / Calulated solar power: $SolarPwrCalc W / Fronius PV: $PVStandby ($FroniusPwr W)"
echo $Logtext
DATE_WITH_TIME=`date "+%Y%m%d-%H%M%S"`
echo $DATE_WITH_TIME " - " $Logtext >> /data/inverter.log
